Type.registerNamespace('FontSize');

FontSize.FontSizeBehavior = function(element) {

    FontSize.FontSizeBehavior.initializeBase(this, [element]);

    // Properties
    this._sizePanelElement = null;
    this._increaseSizeElement = null;
    this._decreaseSizeElement = null;
    this._fontSizeDelta = 2;

    // Behaviors
    this._hoverBehavior = null;
    this._popupBehavior = null;

    // Animations
    this._fadeInAnimation = null;
    this._fadeOutAnimation = null;

    // State
    this._fontSize = 0;
    this._fontUnit = null;
}

FontSize.FontSizeBehavior.prototype = {

    initialize : function() {
        FontSize.FontSizeBehavior.callBaseMethod(this, 'initialize');

        // Initialize HoverBehavior and handlers
        this._hoverBehavior = $create(AjaxControlToolkit.HoverBehavior,
            { id: this.get_id()+"_Hover",
              hoverElement: this._sizePanelElement,
              unhoverDelay: 250 },
            null, null, this.get_element());
        this._hoverBehavior.add_hover(Function.createDelegate(this, this._onHover));
        this._hoverBehavior.add_unhover(Function.createDelegate(this, this._onUnhover));

        // Initialize PopupBehavior
        this._popupBehavior = $create(AjaxControlToolkit.PopupBehavior,
            { id: this.get_id()+"_Popup",
              parentElement: this.get_element(),
              positioningMode: AjaxControlToolkit.PositioningMode.TopRight },
            null, null, this._sizePanelElement);

        // Initialize increment/decrement handlers
        $addHandler(this._increaseSizeElement, "click", Function.createDelegate(this, this._onIncrementFontSize));
        $addHandler(this._decreaseSizeElement, "click", Function.createDelegate(this, this._onDecrementFontSize));

        // Initialize animations
        this._fadeInAnimation = new AjaxControlToolkit.Animation.FadeInAnimation(this._sizePanelElement, 0.25, null, 0.0, 0.75);
        this._fadeOutAnimation = new AjaxControlToolkit.Animation.FadeOutAnimation(this._sizePanelElement, 0.25, null, 0.0, 0.75);
        this._fadeOutAnimation.add_ended(Function.createDelegate(this, this._onFadedOut));

        // Update from client state (if present)
        var clientState = this.get_ClientState();
        if (clientState) {
            this._parseIntoFontSizeAndUnit(clientState);
            this._setFontSize(this._fontSize);
        }
    },

    dispose : function() {
        if (this._hoverBehavior) {
            this._hoverBehavior.dispose();
            this._hoverBehavior = null;
        }
        if (this._popupBehavior) {
            this._popupBehavior.dispose();
            this._popupBehavior = null;
        }
        if (this._fadeInAnimation) {
            this._fadeInAnimation.dispose();
            this._fadeInAnimation = null;
        }
        if (this._fadeOutAnimation) {
            this._fadeOutAnimation.dispose();
            this._fadeOutAnimation = null;
        }

        FontSize.FontSizeBehavior.callBaseMethod(this, 'dispose');
    },

    _onHover : function(eventArgs) {
        // Display the popup
        this._fadeOutAnimation.stop(false);
        this._popupBehavior.show();
        this._fadeInAnimation.play();
    },

    _onUnhover : function(eventArgs) {
        // Remove the popup
        this._fadeInAnimation.stop(false);
        this._fadeOutAnimation.play();
    },

    _onFadedOut : function() {
        // Popup is faded; hide it
        this._popupBehavior.hide();
    },

    _onIncrementFontSize : function(eventArgs) {
        // Increment font size and prevent default handling
        this._setFontSize(this._getFontSize() + this._fontSizeDelta);
        eventArgs.preventDefault();
    },

    _onDecrementFontSize : function(eventArgs) {
        // Decrement font size and prevent default handling
        this._setFontSize(this._getFontSize() - this._fontSizeDelta);
        eventArgs.preventDefault();
    },

    _getFontSize : function() {
        if (!this._fontSize) {
            // Get current fontSize style and store the value
            var currentSize = AjaxControlToolkit.CommonToolkitScripts.getCurrentStyle(this.get_element(), "fontSize");
            this._parseIntoFontSizeAndUnit(currentSize);
        }
        return this._fontSize;
    },

    _setFontSize : function(size) {
        // Update size, keeping it reasonable
        this._fontSize = Math.min(28, Math.max(4, size));

        // Apply the new size
        this.get_element().style.fontSize = this._fontSize + this._fontUnit;

        // Save the size in client state
        this.set_ClientState(this._fontSize + this._fontUnit);
    },

    _parseIntoFontSizeAndUnit : function(value) {
        // Parse it and store the values
        var unit = AjaxControlToolkit.CommonToolkitScripts.parseUnit(value);
        this._fontSize = unit.size;
        this._fontUnit = unit.type;
    },

    get_increaseSizeElement : function() {
        return this._increaseSizeElement;
    },

    set_increaseSizeElement: function(value) {
        this._increaseSizeElement = value;
    },

    get_decreaseSizeElement : function() {
        return this._decreaseSizeElement;
    },

    set_decreaseSizeElement: function(value) {
        this._decreaseSizeElement = value;
    },

    get_fontSizeDelta : function() {
        return this._fontSizeDelta;
    },

    set_fontSizeDelta: function(value) {
        this._fontSizeDelta = value;
    },

    get_sizePanelElement : function() {
        return this._sizePanelElement;
    },

    set_sizePanelElement: function(value) {
        this._sizePanelElement = value;
    }
}

FontSize.FontSizeBehavior.registerClass('FontSize.FontSizeBehavior', AjaxControlToolkit.BehaviorBase);
